package com.ssh;

import android.util.Log;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** SSHPlugin class. **/
public class SshPlugin implements MethodCallHandler, StreamHandler {
  /** Plugin registration. **/
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(
      registrar.messenger(), "ssh"
    );

    final SshPlugin instance = new SshPlugin();
    channel.setMethodCallHandler(instance);
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("start")) {
      start((HashMap) call.arguments, result);
    } else if (call.method.equals("stop")) {
      stop((HashMap) call.arguments, result);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onListen(Object arguments, EventSink events) {}

  @Override
  public void onCancel(Object arguments) {}

  public enum TypeOfAuthentication {
    KEY,
    PASSWORD
  }

  private class SSHClient {
    String           _id;
    Session          _session;
    BufferedReader   _bufferedReader;
    DataOutputStream _dataOutputStream;

    Channel     _channel          = null;
    ChannelSftp _sftpSession      = null;
    Boolean     _downloadContinue = false;
    Boolean     _uploadContinue   = false;
  }

  private static final String LOGTAG = "SSH";

  Map<String, SSHClient> clientPool = new HashMap<>();
  private SSHClient getClient(final String key, final Result result) {
    SSHClient client = clientPool.get(key);
    if (client == null)
      result.error("unknown_client", "Unknown client", null);
    return client;
  }

  public void start(final HashMap args, final Result result) {
    new Thread(new Runnable() {
      public void run() {
        try {
          String id         = args.get("id").toString();
          String tunnelHost = args.get("tunnelHost").toString();
          int    tunnelPort = Integer.parseInt(args.get("tunnelPort").toString());
          String serverHost = args.get("serverHost").toString();
          int    serverPort = Integer.parseInt(args.get("serverPort").toString());
          String remoteHost = args.get("remoteHost").toString();
          int    remotePort = Integer.parseInt(args.get("remotePort").toString());
          String user = args.get("user").toString();

          TypeOfAuthentication typeOfAuthentication = TypeOfAuthentication.values()[Integer.parseInt(args.get("typeOfAuthentication").toString())];
          String password = "";

          JSch jsch = new JSch();
          if (typeOfAuthentication  == TypeOfAuthentication.KEY) {
            HashMap keyPair = (HashMap) args.get("authentication");
            byte[] privateKey = keyPair.containsKey("privateKey") ? keyPair.get("privateKey").toString().getBytes(): null;
            byte[] publicKey = keyPair.containsKey("publicKey") ? keyPair.get("publicKey").toString().getBytes(): null;
            byte[] passphrase = keyPair.containsKey("passphrase") ? keyPair.get("passphrase").toString().getBytes(): null;
            jsch.addIdentity("default", privateKey, publicKey, passphrase);
          } else if (typeOfAuthentication == TypeOfAuthentication.PASSWORD) {
            password = args.get("authentication").toString();
          }

          Session session = jsch.getSession(user, serverHost, serverPort);

          if (password.length() > 0) {
            session.setPassword(password);
          }

          Properties properties = new Properties();
          properties.setProperty("StrictHostKeyChecking", "no");
          session.setConfig(properties);
          session.connect();
          session.setPortForwardingL(tunnelPort, remoteHost, remotePort);

          if (session.isConnected()) {
            SSHClient client = new SSHClient();
            client._session  = session;
            client._id       = id;
            clientPool.put(id, client);

            Log.d(LOGTAG, "Tunnel opened");
            result.success("tunnel_opened");
          }
        } catch (Exception error) {
          Log.e(LOGTAG, "Connection failed: " + error.getMessage());
          result.error("connection_failure", error.getMessage(), null);
        }
      }
    }).start();
  }

  public void stop(final HashMap args, final Result result) {
    SSHClient client = clientPool.get(args.get("id"));
    if (client == null) {
      return;
    }
    client._session.disconnect();
  }
}
